'-------------------------------------------------------------------------------
'-- VBS script file
'-- Created on 
'-- Author:  National Instruments
'-- Comment: Lineare Abbildung fr MDF-Daten
'-------------------------------------------------------------------------------

' Preinitialize the deactivation of linear mapping
GlobalDim("LineareAbbildungAktivieren") : LineareAbbildungAktivieren = false
GlobalDim("PortalAlphabetischSortieren") : PortalAlphabetischSortieren = true


GlobalDim("XRATE") : XRATE = 10

Navigator.Events.OnShowingContextMenu = "MyOnShowingContextMenu"
Navigator.Events.OnContextMenuPointSelected = "MyOnContextMenuPointSelected"
Navigator.Events.OnFileLoading = "MyOnFileLoading"
Navigator.Events.OnLoaded = "MyOnLoaded"

Dim GroupCount_BeforeLoading, GroupCount_AtferLoading

Sub MyOnFileLoading (NaviLoadControl, DataFileName, ImportAction, FileImportFilter, ImportSelection)
  ' While linear mapping the loading of single channels is disabled
  If LineareAbbildungAktivieren Then
    If ImportSelection <> "" AND InStr(1, ImportSelection, "*", 1) = 0 Then
      NaviLoadControl = "SkipLoadCmd"
    End If
  End If
  GroupCount_BeforeLoading = GroupCount
End Sub

Sub MyOnLoaded ()
  GroupCount_AtferLoading = GroupCount
  ' linear mapping, if enabled and used as DataPlugin MDF
  If LineareAbbildungAktivieren AND ((FileImportFilter = "MDF3") or (FileImportFilter = "MDF")) Then
    If GroupCount_BeforeLoading < GroupCount_AtferLoading Then
      Dim ErrorText
      If LinAbb(ErrorText) = false Then
        MsgBox "Error in linear mapping:" & VbCrLf & ErrorText
      End If
      If PortalAlphabetischSortieren Then
        SortPortal GroupCount_BeforeLoading + 1, GroupCount_BeforeLoading + 1, false
      End If
    End If
  End If
End Sub

Sub MyOnShowingContextMenu (ParentObj, MenuPoints)
  ' this is an update for validation of ATF-XML-files
  If (ParentObj.IsKindOf(eDataFinderBrowser)) then
    If (ParentObj.FocusedElement.IsKindOf(eSearchFile)) OR _
       (ParentObj.FocusedElement.IsKindOf("fsifile")) then
      Dim MyArray : MyArray = Split(ParentObj.FocusedElement.Name, ".", -1, 1)
      Dim MyArraySize : MyArraySize = UBound(MyArray, 1)
      If MyArraySize >= 1 then
        Dim Extension : Extension = MyArray(MyArraySize)
        If Extension = "atfx" then
          Call MenuPoints.Add("Validate ATF-XML", 1)          
        End IF
      End If
    End if
  End if

  ' to switch linear mapping on/off
  Call MenuPoints.Add("", 0)          
  If LineareAbbildungAktivieren = true Then
    Call MenuPoints.Add("Linear Mapping On", 2)          
  Else
    Call MenuPoints.Add("Linear Mapping Off", 3)          
  end If
  ' Configurationdialog
  Call MenuPoints.Add("Configure Linear Mapping", 4)          
  ' to switch alphabetic sorting on/off
  If LineareAbbildungAktivieren = true Then
    If PortalAlphabetischSortieren = true Then
      Call MenuPoints.Add("Don't sort channels", 5)
    Else 
      Call MenuPoints.Add("Sort channels", 6)
    End If
  End If
End Sub

Sub MyOnContextMenuPointSelected (ParentObj, MenuPoint)
   Select Case MenuPoint.ID 
      ' this is an update for validation of ATF_XML-files
      Case 1 
        OdsValidateAtfX ParentObj.FocusedElement.Properties("fullpath").Value, 0
        If OdsStat <> 0 Then
          Call WndShow("SCRIPT", "OPEN")
          Call AutEdFileOpen(OdsAtfLogName)
          AutEdTabActivate AutEdNoFiles-1 
        Else
          OdsStatusMsg OdsStat, 1
        End If
      ' to switch linear mapping on/off
      Case 2
        LineareAbbildungAktivieren = false
      Case 3
        LineareAbbildungAktivieren = true
      Case 4
        Dim NewXRATE : NewXRATE = CInt(InputBox("frequency (Hz) for linear mapping: ", "MDF Import: Settings", XRATE))
        if NewXRATE > 0 then 
          XRATE = NewXRATE  
        end if
      Case 5
        PortalAlphabetischSortieren = false
      Case 6
        PortalAlphabetischSortieren = true
   End Select
End Sub


Function LinAbb(LOGTEXT)

  '-------------------------------------------------------------------------------
  '-- define and initialize uservariables
  '---
  LinAbb = true

  ' remember groupname, because it can be changed during creation...
  Dim LinAbb_GroupName : LinAbb_GroupName = Groupcreate("Calculation")
  Call GroupDefaultSet(GroupIndexGet(LinAbb_GroupName))

  ' create the common timechannel
  Dim TimeChannelName : TimeChannelName = CreateCommonTimeChannel(LOGTEXT)
  If TimeChannelName = "" Then
    LinAbb = false
    Exit Function   
  End If
  
  Dim TEXTK : TEXTK = 0
  Dim CurrGroupIndex, CurrChannelIndex, FirstChannelNumber
  'loop at all imported groups
  For CurrGroupIndex = GroupCount_BeforeLoading + 1 to GroupCount_AtferLoading
    'loop at all channels of the group
    FirstChannelNumber = CNoXGet(CurrGroupIndex, 1)
    for CurrChannelIndex = 2 to GroupChnCount(CurrGroupIndex)
       'Channelnumber of the actual channel
      Dim CurrChannelNumber : CurrChannelNumber = CNoXGet(CurrGroupIndex, CurrChannelIndex)
    
      If ChnFormat(CurrChannelNumber)<>"Text" Then
        If ChnLength(CurrChannelNumber)>1 Then
          If Not ( IsNull(Cch(CurrChannelNumber,1)) AND IsNull(Cch(CurrChannelNumber,2)) ) Then
            Call MsgLineDisp("working on channel " & ChnPropGet(CurrChannelNumber, "name"))
            Call ChnMapLinCalc(FirstChannelNumber, CurrChannelNumber, TimeChannelName, "Lin_mapping", 1, "const. value", NOVALUE, "analogue")  '... X,Y,X1,E,MAPLINNOVINTERP,MAPLINEXTTYPE,MAPLINBDRYVAL,MAPLINFCTTYPE
            ' Waveform erzeugen
            Call ChnToWfChn(TimeChannelName, "Lin_mapping", 0)
            Call ChnPropSet("Lin_mapping", "description", ChnPropGet(CurrChannelNumber, "description"))
            Call ChnPropSet("Lin_mapping", "name", ChnPropGet(CurrChannelNumber, "name"))
          End if
        End If
      Else
        If TEXTK=0 Then
          LOGTEXT=LOGTEXT & VbCrLf & "Some textchannels could not be linear mapped !"
          TEXTK=1
          LinAbb = false
        End if
      End if

    next
  next

  ChnDim(TimeChannelName)="s"
  ChnComment(TimeChannelName)="linear mapped"
  ChnName(TimeChannelName)=("Time("+Str(XRATE))+"Hz)"
  
  
  'delete all imported groups
  For CurrGroupIndex = GroupCount_BeforeLoading + 1 to GroupCount_AtferLoading
    Call GroupDel(GroupCount_BeforeLoading + 1)
  Next
  
  GroupName(GroupIndexGet(LinAbb_GroupName)) = NameSplit(DataFileName,"N")    'FirstLoadedGroupName

End Function 'LinAbb


Function CreateCommonTimeChannel(LOGTEXT)
  
  CreateCommonTimeChannel = ""

  Dim XWERTE, XSCHRITTWEITE
  '---
  XWERTE=1
  XSCHRITTWEITE=1

  Dim FirstChannelNumber : FirstChannelNumber = CNoXGet(GroupCount_BeforeLoading + 1, 1)

  ' definition of channeloptions and creating a new, linear timechannel
  Call ChnCharacter(FirstChannelNumber) 'force the calculation of channeloptions...
  Dim MaxTimeValue : MaxTimeValue = ChnPropValGet(FirstChannelNumber, "maximum")
  Dim MinTimeValue : MinTimeValue = ChnPropValGet(FirstChannelNumber, "minimum")

  
  Dim CurrGroupIndex
  Dim MaxTemp
  Dim MinTemp
  'loop at all imported groups and search for timechannel min and max
  For CurrGroupIndex = GroupCount_BeforeLoading + 2 to GroupCount_AtferLoading
    FirstChannelNumber = CNoXGet(CurrGroupIndex, 1)
    Call ChnCharacter(FirstChannelNumber) 'calculation of channeloptions...
    MaxTemp = ChnPropValGet(FirstChannelNumber, "maximum")
    MinTemp = ChnPropValGet(FirstChannelNumber, "minimum")
    If MaxTemp > MaxTimeValue Then
      MaxTimeValue = MaxTemp
    End If
    If MinTemp < MinTimeValue Then
      MinTimeValue = MinTemp
    End If
  Next


  If MinTimeValue > MaxTimeValue Then
    LOGTEXT="The minimum of the timechannel is bigger then the maximum! Program aborted!"
    CreateCommonTimeChannel = false
    Exit Function   
  End If

  XWERTE=Round((MaxTimeValue - MinTimeValue) * XRATE)
  XSCHRITTWEITE=1/XRATE
  
  CreateCommonTimeChannel = ChnLinGenImp("LinTime", XWERTE, MinTimeValue, XSCHRITTWEITE)


End Function


Sub SortPortal(FirstGroup, LastGroup, sortFirstChannel)

  Dim i : For i = FirstGroup to LastGroup
    Call BubbleSort(i,sortFirstChannel)
  Next

End Sub


Sub BubbleSort(groupIndex, sortFirstChn)
  Dim interchanged
  Dim i, n, numberOfChn
  Dim ChnAct, ChnNext
  Dim offset

  If sortFirstChn Then
    offset = 0
  Else
    offset = 1
  End If

  numberOfChn = GroupChnCount(groupIndex)
  n = numberOfChn - 1

  Do
    interchanged = false

    For i = (1 + offset) To n
      ChnAct = ChnName(CNo("[" & groupIndex & "]/[" & i & "]"))
      ChnNext = ChnName(CNo("[" & groupIndex & "]/[" & (i + 1) & "]"))

      If StrComp(ChnAct, ChnNext, 1) = 1 Then
        Call ChnMove(ChnAct, groupIndex, i + 1)
        interchanged = true
      End If
    Next

    n = n - 1
  Loop While interchanged 

End Sub
